(function (ds) {
    ds._o_053.create = options => {
        let _res;
        {
            if (arguments.length === 3) {
                options = {
                    md: arguments[0],
                    mgf: arguments[1],
                    saltLength: arguments[2]
                };
            }
            var hash = options.md;
            var mgf = options.mgf;
            var hLen = hash.digestLength;
            let salt_ = options.salt || null;
            let sLen;
            if (typeof salt_ === 'string') {
                salt_ = ds._o_059._f_125(salt_);
            }
            if ('saltLength' in options) {
                sLen = options.saltLength;
            } else if (salt_ !== null) {
                sLen = salt_.length();
            } else {
                throw new Error('Salt length not specified or specific salt not given.');
            }
            if (salt_ !== null && salt_.length() !== sLen) {
                throw new Error('Given salt length does not match length of given salt.');
            }
            var prng = options.prng || ds._o_055;
            let pssobj = {};
            pssobj.encode = function (md, modBits) {
                let _res;
                {
                    let i;
                    const emBits = modBits - 1;
                    let emLen = Math.ceil(emBits / 8);
                    let mHash = md.digest().getBytes();
                    if (emLen < hLen + sLen + 2) {
                        throw new Error('Message is too long to encrypt.');
                    }
                    let salt;
                    if (salt_ === null) {
                        salt = prng.getBytesSync(sLen);
                    } else {
                        salt = salt_.bytes();
                    }
                    let m_ = new ds._o_059._f_125();
                    m_._f_132(0, 8);
                    m_._f_127(mHash);
                    m_._f_127(salt);
                    hash.start();
                    hash.update(m_.getBytes());
                    let h = hash.digest().getBytes();
                    let ps = new ds._o_059._f_125();
                    ps._f_132(0, emLen - sLen - hLen - 2);
                    ps._f_128(1);
                    ps._f_127(salt);
                    let db = ps.getBytes();
                    let maskLen = emLen - hLen - 1;
                    let dbMask = mgf.generate(h, maskLen);
                    let maskedDB = '';
                    for (i = 0; i < maskLen; i++) {
                        maskedDB += String.fromCharCode(db.charCodeAt(i) ^ dbMask.charCodeAt(i));
                    }
                    let mask = 65280 >> 8 * emLen - emBits & 255;
                    maskedDB = String.fromCharCode(maskedDB.charCodeAt(0) & ~mask) + maskedDB.substr(1);
                    return maskedDB + h + String.fromCharCode(188);
                }
            };
            pssobj.verify = function (mHash, em, modBits) {
                let _res;
                {
                    let i;
                    const emBits = modBits - 1;
                    let emLen = Math.ceil(emBits / 8);
                    em = em.substr(-emLen);
                    if (emLen < hLen + sLen + 2) {
                        throw new Error('Inconsistent parameters to PSS signature verification.');
                    }
                    if (em.charCodeAt(emLen - 1) !== 188) {
                        throw new Error('Encoded message does not end in 0xBC.');
                    }
                    let maskLen = emLen - hLen - 1;
                    let maskedDB = em.substr(0, maskLen);
                    let h = em.substr(maskLen, hLen);
                    let mask = 65280 >> 8 * emLen - emBits & 255;
                    if ((maskedDB.charCodeAt(0) & mask) !== 0) {
                        throw new Error('Bits beyond keysize not zero as expected.');
                    }
                    let dbMask = mgf.generate(h, maskLen);
                    let db = '';
                    for (i = 0; i < maskLen; i++) {
                        db += String.fromCharCode(maskedDB.charCodeAt(i) ^ dbMask.charCodeAt(i));
                    }
                    db = String.fromCharCode(db.charCodeAt(0) & ~mask) + db.substr(1);
                    let checkLen = emLen - hLen - sLen - 2;
                    for (i = 0; i < checkLen; i++) {
                        if (db.charCodeAt(i) !== 0) {
                            throw new Error('Leftmost octets not zero as expected');
                        }
                    }
                    if (db.charCodeAt(checkLen) !== 1) {
                        throw new Error('Inconsistent PSS signature, 0x01 marker not found');
                    }
                    let salt = db.substr(-sLen);
                    let m_ = new ds._o_059._f_125();
                    m_._f_132(0, 8);
                    m_._f_127(mHash);
                    m_._f_127(salt);
                    hash.start();
                    hash.update(m_.getBytes());
                    let h_ = hash.digest().getBytes();
                    return h === h_;
                }
            };
            return pssobj;
        }
    };
}(ds));
